'use strict';

const Homey = require('homey');
var server_data = new Array();
const http = require('http.min');

class myIO_App extends Homey.App {

  async onInit() {
    var string2send;
    var updated_settings= new Array();
    var _settings = this.homey.settings;

    // get settings variables

    var address = new Array();
    var http_port = new Array();
    var username = new Array();
    var password = new Array();
    var defaultSettings = new Array();
    var status = new Array();
    var offline_counter = new Array();
    var serverActive = new Array();

    address[1] = _settings.get('address');
    http_port[1] = _settings.get('http_port');
    username[1] = _settings.get('username');
    password[1] = _settings.get('password');
    defaultSettings[1] = _settings.get('defaultSettings');
    status[1] = _settings.get('status');
    offline_counter[1] = 0;
    serverActive[1] = 'true';
    
    // if it is the first time to start app, fill the settings with default variables
    if (defaultSettings == null) {
      _settings.set('address', '192.168.1.170');
      _settings.set('http_port', '80');
      _settings.set('username', 'admin');
      _settings.set('password', 'admin');
      _settings.set('defaultSettings', 'set');
      _settings.set('status', 'offline');
      _settings.set('serverActive', 'true');
    }  
    _settings.set('updated_settings', 'true');    // set this from the index.htm to true
    _settings.set('server_data', "");
    //Multi server
    for (var i = 2; i <= 10; i++) {
      serverActive[i]=_settings.get('serverActive'+i);
      if (serverActive[i]== 'true') {
        address[i] = _settings.get('address'+i);
        http_port[i] = _settings.get('http_port'+i);
        username[i] = _settings.get('username'+i);
        password[i] = _settings.get('password' + i);
        status[i] = _settings.get('status'+i);
        offline_counter[i] = 0;
        _settings.set('server_data' + i, "");
        _settings.set('updated_settings'+ i, 'true');
      }
    }

    const _this = this;
    var _timeout = 5000;

    await ServerUpdate();

    async function ServerUpdate() {
      var __settings = _this.homey.settings;   
      for (i = 1; i <= 10; i++) {
        var tempNrChar = "";
        if (i > 1) {
          tempNrChar = i.toString();
        }
        updated_settings[i] = __settings.get('updated_settings'+ tempNrChar);

        if (updated_settings[i] == 'true') {                   //if index.html changed the settings or first turn
          __settings.set('server_data' + tempNrChar, "");
          address[i] = __settings.get('address' + tempNrChar);
          http_port[i] = __settings.get('http_port' + tempNrChar);
          username[i] = __settings.get('username' + tempNrChar);
          password[i] = __settings.get('password' + tempNrChar);
          serverActive[i] = __settings.get('serverActive' + tempNrChar);
          _this.log("Server: " + tempNrChar, address, http_port, username, password);
          string2send = "/d_sens_out.json";
          _this.log("Settings_updated");
          __settings.set('updated_settings' + tempNrChar, 'false');
          _timeout = 6000;
        } else {
          string2send = "/sens_out.json";
          _timeout = 1500;
        }
        if (serverActive[i] == 'true') { 
          const _id = i;
          try {
            const respJSON = await http.post({
              uri: "http://" + address[_id] + ":" + http_port[_id] + string2send,
              timeout: _timeout,
              json: true,
              headers: {
                'Authorization': 'Basic ' + Buffer.from(username[_id] + ":" + password[_id]).toString('base64')
              },
              insecureHTTPParser: true
            });
            const inJSON = respJSON.data;
            if (string2send == "/d_sens_out.json" || server_data[_id] == null || server_data[_id] == "") {
              server_data[_id] = inJSON;
            } else if (server_data[_id] != null && server_data[_id] != "") {
              var key, number, element;
              for (key in inJSON) {
                for (number = 0; number < Object.keys(inJSON[key]).length; number++) {
                  for (element in inJSON[key][number]) {
                    server_data[_id][key][number][element] = inJSON[key][number][element];
                  }
                }
              }
            }
            if (server_data[_id] != null && server_data[_id] != "") {
              __settings.set('server_data' + tempNrChar, server_data[_id]);              
              __settings.set('status' + tempNrChar, 'online');
               offline_counter[_id] = 0;
            }
          } catch (e) {
            if (e == 'Error: socket hang up') { }
            else if (e == 'Error: timeout') { }
            else { _this.log('HTTP request error', e); }
            if (offline_counter[_id] == 1) {
              __settings.set('status'+ tempNrChar, 'offline');
              offline_counter[_id] = 0;
            } else { offline_counter[_id] = 1; }
          }
          _this.log("Refreshing Server data:"+_id);
        }
      }
    }
    
    // refreshing the server_data JSON
    setInterval(async() => {                                 
      await ServerUpdate();      
    }, 8000);

    this.log('myIO_App has been initialized');
  }  
  
}


module.exports = myIO_App;
